@@ -18,6 +18,7 @@
 #
 
 import os
+import glob
 import sys
 import yaml
 import codecs
@@ -29,6 +30,7 @@ import optparse
 import time
 import datetime
 import subprocess
+import cgi
 import ansible.utils
 import ansible.utils.module_docs as module_docs
 
@@ -61,11 +63,15 @@ def latex_ify(text):
 
 def html_ify(text):
 
-    t = _ITALIC.sub("<em>" + r"\1" + "</em>", text)
+    #print "DEBUG: text=%s" % text
+    
+    t = cgi.escape(text)
+    t = _ITALIC.sub("<em>" + r"\1" + "</em>", t)
     t = _BOLD.sub("<b>" + r"\1" + "</b>", t)
     t = _MODULE.sub("<span class='module'>" + r"\1" + "</span>", t)
     t = _URL.sub("<a href='" + r"\1" + "'>" + r"\1" + "</a>", t)
     t = _CONST.sub("<code>" + r"\1" + "</code>", t)
+
     return t
 
 def json_ify(text):
@@ -104,9 +110,13 @@ def rst_ify(text):
 
     return t
 
+_MARKDOWN = re.compile(r"[*_`]")
+
 def markdown_ify(text):
 
-    t = _ITALIC.sub("_" + r"\1" + "_", text)
+    t = cgi.escape(text)
+    t = _MARKDOWN.sub(r"\\\g<0>", t)
+    t = _ITALIC.sub("_" + r"\1" + "_", t)
     t = _BOLD.sub("**" + r"\1" + "**", t)
     t = _MODULE.sub("*" + r"\1" + "*", t)
     t = _URL.sub("[" + r"\1" + "](" + r"\1" + ")", t)
@@ -134,9 +144,26 @@ def return_data(text, options, outputname, module):
 
 def boilerplate():
     if not os.path.exists(EXAMPLE_YAML):
-        print >>sys.stderr, "Missing example boiler plate: %S" % EXAMPLE_YAML
+        print >>sys.stderr, "Missing example boiler plate: %s" % EXAMPLE_YAML
+    print "DOCUMENTATION = '''"
     print file(EXAMPLE_YAML).read()
+    print "'''"
+    print ""
 
+def list_modules(module_dir):
+    categories = {}
+    files = glob.glob("%s/*" % module_dir)
+    for d in files:
+        if os.path.isdir(d):
+            files2 = glob.glob("%s/*" % d)
+            for f in files2:
+                tokens = f.split("/")
+                module = tokens[-1]
+                category = tokens[-2]
+                if not category in categories:
+                    categories[category] = {}
+                categories[category][module] = f
+    return categories
 
 def main():
 
@@ -202,6 +229,14 @@ def main():
 
     if options.do_boilerplate:
         boilerplate()
+
+        print ""
+        print "EXAMPLES = '''"
+        print "# example of doing ___ from a playbook"
+        print "your_module: some_arg=1 other_arg=2"
+        print "'''"
+        print ""
+
         sys.exit(0)
 
     if not options.module_dir:
@@ -274,74 +309,114 @@ def main():
 
     # Temporary variable required to genrate aggregated content in 'js' format.
     js_data = []
-    for module in sorted(os.listdir(options.module_dir)):
-        if len(options.module_list):
-            if not module in options.module_list:
+
+    categories = list_modules(options.module_dir)
+    last_category = None
+    category_names = categories.keys()
+    category_names.sort()
+ 
+    for category in category_names:
+        module_map = categories[category]
+ 
+        category = category.replace("_"," ")
+        category = category.title()
+
+        modules = module_map.keys()
+        modules.sort()
+
+        for module in modules:
+
+            print "rendering: %s" % module
+
+            fname = module_map[module]
+
+            if len(options.module_list):
+                if not module in options.module_list:
+                    continue
+
+            # fname = os.path.join(options.module_dir, module)
+
+            extra = os.path.join("inc", "%s.tex" % module)
+
+            # probably could just throw out everything with extensions
+            if fname.endswith(".swp") or fname.endswith(".orig") or fname.endswith(".rej"):
                 continue
 
-        fname = os.path.join(options.module_dir, module)
-        extra = os.path.join("inc", "%s.tex" % module)
+            # print " processing module source ---> %s" % fname
 
-        # probably could just throw out everything with extensions
-        if fname.endswith(".swp") or fname.endswith(".orig") or fname.endswith(".rej"):
-            continue
+            if options.type == 'js':
+                if fname.endswith(".json"):
+                    f = open(fname)
+                    j = json.load(f)
+                    f.close()
+                    js_data.append(j)
+                continue
 
-        print " processing module source ---> %s" % fname
+            doc, examples = ansible.utils.module_docs.get_docstring(fname, verbose=options.verbose)
+
+            if doc is None and module not in ansible.utils.module_docs.BLACKLIST_MODULES:
+                print " while processing module source ---> %s" % fname
+                sys.stderr.write("*** ERROR: CORE MODULE MISSING DOCUMENTATION: %s ***\n" % module)
+                #sys.exit(1)
+
+            if not doc is None:
+ 
+                all_keys = []
+
+                if not 'version_added' in doc:
+                    sys.stderr.write("*** ERROR: missing version_added in: %s ***\n" % module)
+                    sys.exit(1)
+                    if doc['version_added'] == 'historical':
+                       del doc['version_added']
+
+                for (k,v) in doc['options'].iteritems():
+                    all_keys.append(k)
+                all_keys = sorted(all_keys)
+                doc['option_keys'] = all_keys 
+
+                doc['filename']         = fname
+                doc['docuri']           = doc['module'].replace('_', '-')
+                doc['now_date']         = datetime.date.today().strftime('%Y-%m-%d')
+                doc['ansible_version']  = options.ansible_version
+                doc['plainexamples']    = examples  #plain text
+
+                # BOOKMARK: here is where we build the table of contents...
+
+                if options.includes_file is not None and includefmt != "":
+
+                    if last_category != category:
+                         incfile.write("\n\n")
+                         incfile.write(category)
+                         incfile.write("\n")
+                         incfile.write('`' * len(category))
+                         incfile.write("\n\n")
+                         last_category = category
+
+                    incfile.write(includefmt % module)
+
+                if options.verbose:
+                    print json.dumps(doc, indent=4)
+
+
+                if options.type == 'latex':
+                    if os.path.exists(extra):
+                        f = open(extra)
+                        extradata = f.read()
+                        f.close()
+                        doc['extradata'] = extradata
+
+                if options.type == 'json':
+                    text = json.dumps(doc, indent=2)
+                else:
+                    text = template.render(doc)
+
+                return_data(text, options, outputname, module)
 
         if options.type == 'js':
-            if fname.endswith(".json"):
-                f = open(fname)
-                j = json.load(f)
-                f.close()
-                js_data.append(j)
-            continue
-
-        doc, examples = ansible.utils.module_docs.get_docstring(fname, verbose=options.verbose)
-
-        if doc is None and module not in ansible.utils.module_docs.BLACKLIST_MODULES:
-            sys.stderr.write("*** ERROR: CORE MODULE MISSING DOCUMENTATION: %s ***\n" % module)
-            #sys.exit(1)
-
-        if not doc is None:
-
-            all_keys = []
-            for (k,v) in doc['options'].iteritems():
-                all_keys.append(k)
-            all_keys = sorted(all_keys)
-            doc['option_keys'] = all_keys 
-
-            doc['filename']         = fname
-            doc['docuri']           = doc['module'].replace('_', '-')
-            doc['now_date']         = datetime.date.today().strftime('%Y-%m-%d')
-            doc['ansible_version']  = options.ansible_version
-            doc['plainexamples']    = examples  #plain text
-
-            if options.includes_file is not None and includefmt != "":
-                incfile.write(includefmt % module)
-
-            if options.verbose:
-                print json.dumps(doc, indent=4)
-
-
-            if options.type == 'latex':
-                if os.path.exists(extra):
-                    f = open(extra)
-                    extradata = f.read()
-                    f.close()
-                    doc['extradata'] = extradata
-
-            if options.type == 'json':
-                text = json.dumps(doc, indent=2)
-            else:
-                text = template.render(doc)
-
-            return_data(text, options, outputname, module)
-
-    if options.type == 'js':
-        docs = {}
-        docs['json'] = json.dumps(js_data, indent=2)
-        text = template.render(docs)
-        return_data(text, options, outputname, 'modules')
+            docs = {}
+            docs['json'] = json.dumps(js_data, indent=2)
+            text = template.render(docs)
+            return_data(text, options, outputname, 'modules')
 
 if __name__ == '__main__':
     main()
